@@ -15,6 +15,8 @@
 *                                                                         *
 ***************************************************************************/
 
+#include "kimagemapeditor.h"
+
 #include <iostream>
 #include <assert.h>
 
@@ -22,13 +24,13 @@
 #include <QApplication>
 #include <QAction>
 #include <QDialogButtonBox>
-#include <qlayout.h>
-#include <qpushbutton.h>
-#include <qcombobox.h>
-#include <qfontdatabase.h>
-#include <qfile.h>
+#include <QLayout>
+#include <QPushButton>
+#include <QComboBox>
+#include <QFontDatabase>
+#include <QFile>
 #include <QFileDialog>
-#include <qfileinfo.h>
+#include <QFileInfo>
 #include <QIcon>
 #include <QInputDialog>
 #include <QLinkedList>
@@ -36,16 +38,16 @@
 #include <QMenu>
 #include <QMimeDatabase>
 #include <QMimeType>
-#include <qpixmap.h>
-#include <qpainter.h>
+#include <QPixmap>
+#include <QPainter>
 #include <QPushButton>
 #include <QScrollArea>
-#include <qsplitter.h>
+#include <QSplitter>
 #include <QStandardPaths>
-#include <qtabwidget.h>
+#include <QTabWidget>
 #include <QTextEdit>
-#include <qtextstream.h>
-#include <qtooltip.h>
+#include <QTextStream>
+#include <QToolTip>
 #include <QUndoStack>
 #include <QVBoxLayout>
 
@@ -68,8 +70,6 @@
 #include <KConfigGroup>
 
 // local
-#include "kimagemapeditor.h"
-#include "kimagemapeditor.moc"
 #include "drawzone.h"
 #include "kimedialogs.h"
 #include "kimecommands.h"
@@ -149,28 +149,28 @@ KImageMapEditor::KImageMapEditor(QWidget *parentWidget,
            this,
            SLOT(showTagEditor(QTreeWidgetItem*)));
   connect( areaListView->listView,
-           SIGNAL(customContextMenuRequested(const QPoint&)),
+           SIGNAL(customContextMenuRequested(QPoint)),
            this,
-           SLOT(slotShowPopupMenu(const QPoint &)));
+           SLOT(slotShowPopupMenu(QPoint)));
 
-  connect( mapsListView, SIGNAL( mapSelected(const QString &)),
-           this, SLOT( setMap(const QString &)));
+  connect( mapsListView, SIGNAL(mapSelected(QString)),
+           this, SLOT(setMap(QString)));
 
-  connect( mapsListView, SIGNAL( mapRenamed(const QString &)),
-           this, SLOT( setMapName(const QString &)));
+  connect( mapsListView, SIGNAL(mapRenamed(QString)),
+           this, SLOT(setMapName(QString)));
 
   connect( mapsListView->listView(),
-           SIGNAL(customContextMenuRequested(const QPoint &)),
+           SIGNAL(customContextMenuRequested(QPoint)),
            this,
-           SLOT(slotShowMapPopupMenu(const QPoint &)));
+           SLOT(slotShowMapPopupMenu(QPoint)));
 
-  connect( imagesListView, SIGNAL( imageSelected(const KUrl &)),
-           this, SLOT( setPicture(const KUrl &)));
+  connect( imagesListView, &ImagesListView::imageSelected,
+           this, QOverload<const QUrl &>::of(&KImageMapEditor::setPicture));
 
   connect( imagesListView,
-           SIGNAL(customContextMenuRequested(const QPoint &)),
+           SIGNAL(customContextMenuRequested(QPoint)),
            this,
-           SLOT(slotShowImagePopupMenu(const QPoint &)));
+           SLOT(slotShowImagePopupMenu(QPoint)));
 
   if (splitter) {
     drawZone = new DrawZone(splitter,this);
@@ -401,7 +401,7 @@ void KImageMapEditor::slotConfigChanged()
 }
 
 void KImageMapEditor::openLastURL(const KConfigGroup & config) {
-  KUrl lastURL ( config.readPathEntry("lastopenurl", QString()) );
+  QUrl lastURL ( config.readPathEntry("lastopenurl", QString()) );
   QString lastMap = config.readEntry("lastactivemap");
   QString lastImage = config.readPathEntry("lastactiveimage", QString());
 
@@ -446,7 +446,7 @@ void KImageMapEditor::setupActions()
   temp->setToolTip(i18n("Open new picture or HTML file"));
 
   // File Open Recent
-  recentFilesAction = KStandardAction::openRecent(this, SLOT(openURL(const QUrl&)),
+  recentFilesAction = KStandardAction::openRecent(this, SLOT(openURL(QUrl)),
                                       actionCollection());
 	// File Save
   temp =KStandardAction::save(this, SLOT(fileSave()), actionCollection());
@@ -485,7 +485,7 @@ void KImageMapEditor::setupActions()
   deleteAction = new QAction(QIcon::fromTheme(QStringLiteral("edit-delete")),
       i18n("&Delete"), this);
   actionCollection()->addAction("edit_delete", deleteAction );
-  connect(deleteAction, SIGNAL(triggered(bool) ), SLOT (slotDelete()));
+  connect(deleteAction, SIGNAL(triggered(bool)), SLOT (slotDelete()));
   actionCollection()->setDefaultShortcut(deleteAction, QKeySequence(Qt::Key_Delete));
   deleteAction->setWhatsThis(i18n("<h3>Delete</h3>"
                           "Click this to <em>delete</em> the selected area."));
@@ -653,64 +653,64 @@ void KImageMapEditor::setupActions()
 
     QAction *action  = new QAction(i18n("Cancel Drawing"), this);
     actionCollection()->addAction("canceldrawing", action );
-  connect(action, SIGNAL(triggered(bool)), SLOT( slotCancelDrawing() ));
+  connect(action, SIGNAL(triggered(bool)), SLOT(slotCancelDrawing()));
   actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::Key_Escape));
 
   moveLeftAction  = new QAction(i18n("Move Left"), this);
   actionCollection()->addAction("moveleft", moveLeftAction );
   connect(moveLeftAction, SIGNAL(triggered(bool)),
-         SLOT( slotMoveLeft() ));
+         SLOT(slotMoveLeft()));
   actionCollection()->setDefaultShortcut(moveLeftAction, QKeySequence(Qt::Key_Left));
 
     moveRightAction  = new QAction(i18n("Move Right"), this);
     actionCollection()->addAction("moveright", moveRightAction );
-  connect(moveRightAction, SIGNAL(triggered(bool)), SLOT( slotMoveRight() ));
+  connect(moveRightAction, SIGNAL(triggered(bool)), SLOT(slotMoveRight()));
   actionCollection()->setDefaultShortcut(moveRightAction, QKeySequence(Qt::Key_Right));
 
     moveUpAction  = new QAction(i18n("Move Up"), this);
     actionCollection()->addAction("moveup", moveUpAction );
-  connect(moveUpAction, SIGNAL(triggered(bool)), SLOT( slotMoveUp() ));
+  connect(moveUpAction, SIGNAL(triggered(bool)), SLOT(slotMoveUp()));
   actionCollection()->setDefaultShortcut(moveUpAction, QKeySequence(Qt::Key_Up));
 
     moveDownAction  = new QAction(i18n("Move Down"), this);
     actionCollection()->addAction("movedown", moveDownAction );
-  connect(moveDownAction, SIGNAL(triggered(bool)), SLOT( slotMoveDown() ));
+  connect(moveDownAction, SIGNAL(triggered(bool)), SLOT(slotMoveDown()));
   actionCollection()->setDefaultShortcut(moveDownAction, QKeySequence(Qt::Key_Down));
 
     increaseWidthAction  = new QAction(i18n("Increase Width"), this);
     actionCollection()->addAction("increasewidth", increaseWidthAction );
-  connect(increaseWidthAction, SIGNAL(triggered(bool)), SLOT( slotIncreaseWidth() ));
+  connect(increaseWidthAction, SIGNAL(triggered(bool)), SLOT(slotIncreaseWidth()));
   actionCollection()->setDefaultShortcut(increaseWidthAction, QKeySequence(Qt::Key_Right + Qt::SHIFT));
 
     decreaseWidthAction  = new QAction(i18n("Decrease Width"), this);
     actionCollection()->addAction("decreasewidth", decreaseWidthAction );
-  connect(decreaseWidthAction, SIGNAL(triggered(bool)), SLOT( slotDecreaseWidth() ));
+  connect(decreaseWidthAction, SIGNAL(triggered(bool)), SLOT(slotDecreaseWidth()));
   actionCollection()->setDefaultShortcut(decreaseWidthAction, QKeySequence(Qt::Key_Left + Qt::SHIFT));
 
     increaseHeightAction  = new QAction(i18n("Increase Height"), this);
     actionCollection()->addAction("increaseheight", increaseHeightAction );
-  connect(increaseHeightAction, SIGNAL(triggered(bool)), SLOT( slotIncreaseHeight() ));
+  connect(increaseHeightAction, SIGNAL(triggered(bool)), SLOT(slotIncreaseHeight()));
   actionCollection()->setDefaultShortcut(increaseHeightAction, QKeySequence(Qt::Key_Up + Qt::SHIFT));
 
     decreaseHeightAction  = new QAction(i18n("Decrease Height"), this);
     actionCollection()->addAction("decreaseheight", decreaseHeightAction );
-  connect(decreaseHeightAction, SIGNAL(triggered(bool)), SLOT( slotDecreaseHeight() ));
+  connect(decreaseHeightAction, SIGNAL(triggered(bool)), SLOT(slotDecreaseHeight()));
   actionCollection()->setDefaultShortcut(decreaseHeightAction, QKeySequence(Qt::Key_Down + Qt::SHIFT));
 
     toFrontAction  = new QAction(i18n("Bring to Front"), this);
     actionCollection()->addAction("tofront", toFrontAction );
-  connect(toFrontAction, SIGNAL(triggered(bool)), SLOT( slotToFront() ));
+  connect(toFrontAction, SIGNAL(triggered(bool)), SLOT(slotToFront()));
 
     toBackAction  = new QAction(i18n("Send to Back"), this);
     actionCollection()->addAction("toback", toBackAction );
-  connect(toBackAction, SIGNAL(triggered(bool)), SLOT( slotToBack() ));
+  connect(toBackAction, SIGNAL(triggered(bool)), SLOT(slotToBack()));
 
     forwardOneAction  = new QAction(QIcon::fromTheme(QStringLiteral("raise")), i18n("Bring Forward One"), this);
     actionCollection()->addAction("forwardone", forwardOneAction );
-  connect(forwardOneAction, SIGNAL(triggered(bool) ), SLOT( slotForwardOne() ));
+  connect(forwardOneAction, SIGNAL(triggered(bool)), SLOT(slotForwardOne()));
     backOneAction  = new QAction(QIcon::fromTheme(QStringLiteral("lower")), i18n("Send Back One"), this);
     actionCollection()->addAction("backone", backOneAction );
-  connect(backOneAction, SIGNAL(triggered(bool) ), SLOT( slotBackOne() ));
+  connect(backOneAction, SIGNAL(triggered(bool)), SLOT(slotBackOne()));
 
   areaListView->upBtn->addAction(forwardOneAction);
   areaListView->downBtn->addAction(backOneAction);
@@ -720,7 +720,7 @@ void KImageMapEditor::setupActions()
 
     action  = new QAction(QIcon::fromTheme(QStringLiteral("configure")), i18n("Configure KImageMapEditor..."), this);
     actionCollection()->addAction("configure_kimagemapeditor", action );
-  connect(action, SIGNAL(triggered(bool) ), SLOT(slotShowPreferences()));
+  connect(action, SIGNAL(triggered(bool)), SLOT(slotShowPreferences()));
 
   qCDebug(KIMAGEMAPEDITOR_LOG) << "KImageMapEditor: 1";
 
@@ -1435,7 +1435,7 @@ void KImageMapEditor::setMapName(const QString & s) {
 }
 
 
-void KImageMapEditor::setPicture(const KUrl & url) {
+void KImageMapEditor::setPicture(const QUrl & url) {
   _imageUrl=url;
   if (QFileInfo(url.path()).exists()) {
      QImage img(url.path());
@@ -1591,7 +1591,7 @@ void KImageMapEditor::mapShowHTML()
   delete dialog;
 }
 
-void KImageMapEditor::openFile(const KUrl & url) {
+void KImageMapEditor::openFile(const QUrl & url) {
   if ( ! url.isEmpty()) {
     QMimeDatabase db;
     QMimeType openedFileType = db.mimeTypeForUrl(url);
@@ -1621,7 +1621,7 @@ void KImageMapEditor::fileOpen() {
                      i18n("Web File (*.png *.jpg *.jpeg *.gif *.htm *.html);;Images (*.png *.jpg *.jpeg *.gif *.bmp *.xbm *.xpm *.pnm *.mng);;"
                           "HTML Files (*.htm *.html);;All Files (*)"));
 
-  openFile(KUrl( fileName ));
+  openFile(QUrl::fromUserInput( fileName ));
 }
 
 
@@ -1656,7 +1656,7 @@ void KImageMapEditor::fileSave()
 
 void KImageMapEditor::fileSaveAs() {
 
-  KUrl url = QFileDialog::getSaveFileUrl(widget(), QString(), QUrl(), i18n("HTML File (*.htm *.html);;Text File (*.txt);;All Files (*)" ));
+  QUrl url = QFileDialog::getSaveFileUrl(widget(), QString(), QUrl(), i18n("HTML File (*.htm *.html);;Text File (*.txt);;All Files (*)" ));
   if (url.isEmpty() || !url.isValid()) {
     return;
   }
@@ -1664,20 +1664,6 @@ void KImageMapEditor::fileSaveAs() {
 
   QFileInfo fileInfo(url.path());
 
-  if ( fileInfo.exists() )
-  {
-  	if (KMessageBox::warningContinueCancel(widget(),
-      i18n("<qt>The file <em>%1</em> already exists.<br />Do you want to overwrite it?</qt>", fileInfo.fileName()),
-      i18n("Overwrite File?"), KGuiItem(i18n("Overwrite")))==KMessageBox::Cancel)
-      return;
-
-    if(!fileInfo.isWritable()) {
-      KMessageBox::sorry(widget(), i18n("<qt>You do not have write permission for the file <em>%1</em>.</qt>", fileInfo.fileName()));
-      return;
-    }
-  }
-
-
   saveAs(url);
   recentFilesAction->addUrl(url);
 
@@ -1686,7 +1672,7 @@ void KImageMapEditor::fileSaveAs() {
 
 bool KImageMapEditor::openFile()
 {
-  KUrl u = url();
+  QUrl u = url();
   QFileInfo fileInfo(u.path());
 
   if ( !fileInfo.exists() )
@@ -1806,9 +1792,10 @@ QHash<QString,QString> KImageMapEditor::getTagAttributes(QTextStream & s, QStrin
     } else
     // finished ?
     if (w=='>') {
-      if (valueRead)
+      if (valueRead) {
         dict.insert(attr,value);
-        return dict;
+      }
+      return dict;
     } else
     // currently reading an attribute ?
     if (attrRead) {
@@ -1872,7 +1859,7 @@ QHash<QString,QString> KImageMapEditor::getTagAttributes(QTextStream & s, QStrin
 }
 
 
-bool KImageMapEditor::openHTMLFile(const KUrl & url)
+bool KImageMapEditor::openHTMLFile(const QUrl & url)
 {
   QFile f(url.path());
   if ( !f.exists () )
@@ -1956,7 +1943,7 @@ bool KImageMapEditor::openHTMLFile(const KUrl & url)
 
   f.close();
 
-  KUrl imageUrl;
+  QUrl imageUrl;
 
   map = 0L;
 
@@ -1969,8 +1956,14 @@ bool KImageMapEditor::openHTMLFile(const KUrl & url)
     if (images.count() > 1) {
       ImageTag* imgTag = images.first();
       if (imgTag) {
-        if (imgTag->contains("src"))
-          imageUrl = KUrl(url,imgTag->value("src"));
+        if (imgTag->contains("src")) {
+            if (url.path().isEmpty() | !url.path().endsWith('/')) {
+                imageUrl = QUrl(url.path() + '/').resolved(QUrl(imgTag->value("src")));
+            }
+            else {
+                imageUrl = url.resolved(QUrl(imgTag->value("src")));
+            }
+        }
       }
     }
 
@@ -1988,7 +1981,12 @@ bool KImageMapEditor::openHTMLFile(const KUrl & url)
                 QString usemapName = usemap.right(usemap.length()-1);
                 if (usemapName == mapTag->name) {
 		  if (imageTag->contains("src")) {
-                      imageUrl = KUrl(url,imageTag->value("src"));
+                      if (url.path().isEmpty() | !url.path().endsWith('/')) {
+                          imageUrl = QUrl(url.path() + '/').resolved(QUrl(imageTag->value("src")));
+                      }
+                      else {
+                          imageUrl = url.resolved(QUrl(imageTag->value("src")));
+                      }
 		      found = true;
 		  }
                 }
@@ -2330,14 +2328,14 @@ QString KImageMapEditor::getHtmlCode() {
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 */
-static KUrl toRelative(const KUrl& urlToConvert,const KUrl& baseURL)
+static QUrl toRelative(const QUrl& urlToConvert,const QUrl& baseURL)
 {
-  KUrl resultURL = urlToConvert;
-  if (urlToConvert.protocol() == baseURL.protocol())
+  QUrl resultURL = urlToConvert;
+  if (urlToConvert.scheme() == baseURL.scheme())
   {
     QString path = urlToConvert.path();
-    QString basePath = baseURL.path(KUrl::AddTrailingSlash);
-    if (path.startsWith("/"))
+    QString basePath = baseURL.path().endsWith('/') ? baseURL.path() : baseURL.path() + '/';
+    if (path.startsWith("/") && basePath != "/")
     {
       path.remove(0, 1);
       basePath.remove(0, 1);
@@ -2370,16 +2368,16 @@ static KUrl toRelative(const KUrl& urlToConvert,const KUrl& baseURL)
     resultURL.setPath(QDir::cleanPath(path));
   }
 
-  if (urlToConvert.path().endsWith('/')) resultURL.adjustPath(KUrl::AddTrailingSlash);
+  if (urlToConvert.path().endsWith('/')) resultURL.setPath(resultURL.path() + '/');
   return resultURL;
 }
 
 
-void KImageMapEditor::saveImageMap(const KUrl & url)
+void KImageMapEditor::saveImageMap(const QUrl & url)
 {
   QFileInfo fileInfo(url.path());
 
-  if (!QFileInfo(url.directory()).isWritable()) {
+  if (!QFileInfo(url.adjusted(QUrl::RemoveFilename|QUrl::StripTrailingSlash).path()).isWritable()) {
     KMessageBox::error(widget(),
       i18n("<qt>The file <i>%1</i> could not be saved, because you do not have the required write permissions.</qt>", url.path()));
     return;
@@ -2387,7 +2385,7 @@ void KImageMapEditor::saveImageMap(const KUrl & url)
 
   if (!backupFileCreated) {
     QString backupFile = url.path()+'~';
-    KIO::file_copy(url, KUrl(backupFile ), -1, KIO::Overwrite | KIO::HideProgressInfo);
+    KIO::file_copy(url, QUrl::fromUserInput(backupFile ), -1, KIO::Overwrite | KIO::HideProgressInfo);
     backupFileCreated = true;
   }
 
@@ -2409,7 +2407,7 @@ void KImageMapEditor::saveImageMap(const KUrl & url)
       << "<body>\n"
       << "  " << getHTMLImageMap()
       << "\n"
-      << "  <img src=\"" << toRelative(_imageUrl,KUrl( url.directory() )).path() << "\""
+      << "  <img src=\"" << toRelative(_imageUrl,QUrl( url.adjusted(QUrl::RemoveFilename|QUrl::StripTrailingSlash).path() )).path() << "\""
       << " usemap=\"#" << _mapName << "\""
       << " width=\"" << drawZone->picture().width() << "\""
       << " height=\"" << drawZone->picture().height() << "\">\n"
@@ -2787,11 +2785,11 @@ bool KImageMapEditor::closeUrl()
 
 }
 
-void KImageMapEditor::addImage(const KUrl & imgUrl) {
+void KImageMapEditor::addImage(const QUrl & imgUrl) {
     if (imgUrl.isEmpty())
         return;
 
-    QString relativePath ( toRelative(imgUrl, KUrl( url().adjusted(QUrl::RemoveFilename).path() )).path() );
+    QString relativePath ( toRelative(imgUrl, QUrl( url().adjusted(QUrl::RemoveFilename).path() )).path() );
 
     QString imgHtml = QString("<img src=\"")+relativePath+QString("\">");
     ImageTag* imgTag = new ImageTag();
@@ -2830,7 +2828,7 @@ void KImageMapEditor::setImageActionsEnabled(bool b) {
 
 
 void KImageMapEditor::imageAdd() {
-    KUrl imgUrl = QFileDialog::getOpenFileUrl(widget(), i18n("Select image"),
+    QUrl imgUrl = QFileDialog::getOpenFileUrl(widget(), i18n("Select image"),
                   QUrl(), i18n("Images (*.png *.jpg *.jpeg *.gif *.bmp *.xbm *.xpm *.pnm *.mng);;All Files (*)"));
     addImage(imgUrl);
 }
@@ -2849,7 +2847,7 @@ void KImageMapEditor::imageRemove() {
       ImageTag* selected = imagesListView->selectedImage();
       if (selected) {
 	if (selected->contains("src")) {
-	  setPicture(KUrl(selected->value("src")));
+	  setPicture(QUrl(selected->value("src")));
 	}
       }
     }
